Imports System.Data.Entity
Imports System.Linq.Expressions
Imports TreeDomainClasses
Imports DataModel

Namespace TreesMVC.Models
  Public Class LeafRepository
	  Implements ILeafRepository
	Private context As New TreeContext

	Public ReadOnly Property All As IQueryable(Of Leaf) Implements ILeafRepository.All
	  Get
		  Return context.Leaves
	  End Get
	End Property

	Public Function AllIncluding(ParamArray ByVal includeProperties() As Expression(Of Func(Of Leaf, Object))) As IQueryable(Of Leaf) Implements ILeafRepository.AllIncluding
	  Dim query As IQueryable(Of Leaf) = context.Leaves
	  For Each includeProperty In includeProperties
		query = query.Include(includeProperty)
	  Next includeProperty
	  Return query
	End Function

	Public Function Find(ByVal id As Integer) As Leaf Implements ILeafRepository.Find
	  Return context.Leaves.Find(id)
	End Function

	Public Sub InsertOrUpdate(ByVal leaf As Leaf, ByVal treeId? As Integer) Implements ILeafRepository.InsertOrUpdate
            If leaf.LeafId = 0 Then
                'new leaf
                Dim tree = context.Trees.Find(treeId)
                tree.Leaves.Add(leaf)

            Else
                ' Existing leaf
                context.Entry(leaf).State = EntityState.Modified
            End If
	End Sub

	Public Sub Delete(ByVal id As Integer) Implements ILeafRepository.Delete
	  Dim leaf = context.Leaves.Find(id)
	  context.Leaves.Remove(leaf)
	End Sub

	Public Sub Save() Implements ILeafRepository.Save
	  context.SaveChanges()
	End Sub
  End Class

  Public Interface ILeafRepository
	ReadOnly Property All As IQueryable(Of Leaf)
	Function AllIncluding(ParamArray ByVal includeProperties() As Expression(Of Func(Of Leaf, Object))) As IQueryable(Of Leaf)
	Function Find(ByVal id As Integer) As Leaf
	Sub InsertOrUpdate(ByVal leaf As Leaf, ByVal treeID? As Integer)
	Sub Delete(ByVal id As Integer)
	Sub Save()
  End Interface
End Namespace