Imports System.Data.Entity
Imports System.Linq.Expressions
Imports TreeDomainClasses
Imports DataModel

Namespace TreesMVC.Models
  Public Class TreeRepository
	  Implements ITreeRepository
	Private context As New TreeContext

	Public ReadOnly Property All As IQueryable(Of Tree) Implements ITreeRepository.All
	  Get
		  Return context.Trees
	  End Get
	End Property

	Public Function AllIncluding(ParamArray ByVal includeProperties() As Expression(Of Func(Of Tree, Object))) As IQueryable(Of Tree) Implements ITreeRepository.AllIncluding

	  Dim query As IQueryable(Of Tree) = context.Trees
	  For Each includeProperty In includeProperties
		query = query.Include(includeProperty)
	  Next includeProperty
	  Return query
	End Function

	'Since Leaf doesn't have a TreeId, you can't query Leaves.
	'Instead we'll query Trees but return Leaves.
	'Select returns a list of a collection of leaves
	'FirstOrDefault gets the first (and only in this case) collection        
	Public Function LeavesForTree(ByVal id As Integer) As ICollection(Of Leaf)
	  Return context.Trees.Where(Function(t) t.TreeId = id).Select(Function(t) t.Leaves).FirstOrDefault()
	End Function


	Public Function Find(ByVal id As Integer) As Tree Implements ITreeRepository.Find
	  Return context.Trees.Find(id)
	End Function

	Public Sub InsertOrUpdate(ByVal tree As Tree) Implements ITreeRepository.InsertOrUpdate
            If tree.TreeId = 0 Then
                ' New tree
                If tree.Photo Is Nothing Then
                    tree.Photo = New TreePhoto With {.Photo = New Byte() {0}, .Caption = "No Photo Yet"}
                End If
                context.Trees.Add(tree)
            Else
                context.Entry(tree).State = EntityState.Modified
                context.Entry(tree).Reference(Function(t) t.Photo).Load()
            End If
	End Sub

	Public Sub Delete(ByVal id As Integer) Implements ITreeRepository.Delete
	  Dim tree = context.Trees.Find(id)
	  context.Trees.Remove(tree)
	End Sub

	Public Sub Save() Implements ITreeRepository.Save
	  context.SaveChanges()
	End Sub
  End Class

  Public Interface ITreeRepository
	ReadOnly Property All As IQueryable(Of Tree)
	Function AllIncluding(ParamArray ByVal includeProperties() As Expression(Of Func(Of Tree, Object))) As IQueryable(Of Tree)
	Function Find(ByVal id As Integer) As Tree
	Sub InsertOrUpdate(ByVal tree As Tree)
	Sub Delete(ByVal id As Integer)
	Sub Save()
  End Interface
End Namespace